
  library ieee;
  use ieee.std_logic_1164.ALL;
  use ieee.numeric_std.ALL;

  entity testbench is
   generic(
						size : positive := 8
			 );
  end testbench;

  architecture behavior of testbench is 

		component power
		 port(
            clock 			: in 	std_logic;
            base, exponent : in 	std_logic_vector(size-1 downto 0);       
				result 			: out std_logic_vector(size-1 downto 0);
				ready          : out std_logic;
				enable         : in  std_logic;
				valid          : out std_logic
           );
      end component;

      signal signal_base 		: std_logic_vector(size-1 downto 0);
      signal signal_exponent 	: std_logic_vector(size-1 downto 0);
		signal signal_result		: std_logic_vector(size-1 downto 0);
		signal signal_ready     : std_logic;
		signal signal_valid     : std_logic;
		signal signal_enable    : std_logic;
		signal clock_internal 	: std_logic;
			
		-- Clock period definitions
		constant clock_period_testbench		: time := 10ns;
		signal stop_the_clock: boolean;
  begin 
          uut: power port map
			 (
                  base 		=> signal_base,
                  exponent => signal_exponent,
						result	=> signal_result,
						clock 	=> clock_internal,
						ready    => signal_ready,
						enable   => signal_enable,
						valid    => signal_valid
          );

	clocking : process
	begin
	  while not stop_the_clock loop
	    clock_internal <= '1', '0' after clock_period_testbench / 2;
		 wait for clock_period_testbench;
	  end loop;
	  wait;
	end process;

   tb : process
   begin
		wait until signal_ready = '1';
		-- power(0,0) mod 256 = 1
		signal_base 		<= "00000000";
		signal_exponent 	<= "00000000";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "00000001" report "wrong result for power(0,0)";
		------------------------------------------------------------
		wait until signal_ready = '1';
		-- power(3,0) mod 256 = 1
		signal_base 		<= "00000011";
		signal_exponent 	<= "00000000";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "00000001" report "wrong result for power(3,0)";
		------------------------------------------------------------
		wait until signal_ready = '1';
		-- power(3,1) mod 256 = 3
		signal_base 		<= "00000011";
		signal_exponent 	<= "00000001";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "00000011" report "wrong result for power(3,1)";	
		-------------------------------------------------------
		wait until signal_ready = '1';
		-- power(0,3) mod 256 = 0
		signal_base 		<= "00000000";
		signal_exponent 	<= "00000011";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "00000000" report "wrong result for power(0,3)";	
		-------------------------------------------------------
		wait until signal_ready = '1';
		-- power(3,4) mod 256 = 81
		signal_base 		<= "00000011";
		signal_exponent 	<= "00000100";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "01010001" report "wrong result for power(3,3)";	
		-----------------------------------------------
		wait until signal_ready = '1';		
		-- power (13,3) mod 256 = 149
		signal_base 		<= "00001101";
		signal_exponent 	<= "00000011";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "10010101" report "wrong result for power(13,3)";
		-----------------------------------------------
		wait until signal_ready = '1';		
		-- power (255,2) mod 256 = 1
		signal_base 		<= "11111111";
		signal_exponent 	<= "00000010";
		signal_enable 		<= '1';
		wait until signal_valid = '1';
		signal_enable     <= '0';
		assert signal_result = "00000001" report "wrong result for power(255,2)";
		-- will wait forever
      wait; 
   end process tb;

  end;
